<?php

namespace Myquest\Controller;

use Myquest\Service\JWT;
use Myquest\Util\ConnectionCustom;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class LoginController extends Simple
{

    protected $entityManager;
    protected $currentSession;
    protected $environment;
    protected $connectionCustom;

    public function __construct($entityManager, $currentSession, $environment, $connectionCustom, $loginService, $fileServerFilesPath)
    {
        $this->entityManager = $entityManager;
        $this->currentSession = $currentSession;
        $this->environment = $environment;
        $this->connectionCustom = $connectionCustom;
        $this->loginService = $loginService;
        $this->fileServerFilesPath = $fileServerFilesPath;
    }

    public function getUserByEmailAndPass($params, $NRORG = null, $CDOPERADOR = null)
    {
        $conn = $this->entityManager->getConnection();
        $params['NRORG'] = $NRORG;
        $params['CDOPERADOR'] = $CDOPERADOR;

        $statement = $conn->prepare("SELECT O.DSEMAILOPER, O.NMOPERADOR, O.CDOPERADOR, O.CDFILIALSELECIONADA, O.NRORG, O.DSIMAGEMOPER, F.NMFILIAL FROM OPERADOR O, FILIAL F WHERE F.CDFILIAL (+) = O.CDFILIALSELECIONADA AND DSEMAILOPER =:DSEMAILOPER AND CDSENHAOPERWEB =:CDSENHAOPERWEB AND IDOPERATIVO = 'A' AND NVL(:NRORG,O.NRORG) = O.NRORG AND NVL(:CDOPERADOR,O.CDOPERADOR) = O.CDOPERADOR");
        $statement->execute($params);

        return $statement->fetchAll();
    }
    
    public function getUserByAccessNameAndPass($params, $NRORG = null, $CDOPERADOR = null){
        $conn = $this->entityManager->getConnection();
        $params['NRORG'] = $NRORG;
        $params['CDOPERADOR'] = $CDOPERADOR;

        $statement = $conn->prepare("SELECT O.DSEMAILOPER, O.NMOPERADOR, O.CDOPERADOR, O.CDFILIALSELECIONADA, O.NRORG, O.DSIMAGEMOPER, F.NMFILIAL FROM OPERADOR O, FILIAL F WHERE F.CDFILIAL (+) = O.CDFILIALSELECIONADA AND NMACESSO =:NMACESSO AND CDSENHAOPERWEB =:CDSENHAOPERWEB AND IDOPERATIVO = 'A' AND NVL(:NRORG,O.NRORG) = O.NRORG AND NVL(:CDOPERADOR,O.CDOPERADOR) = O.CDOPERADOR");
        $statement->execute($params);

        return $statement->fetchAll();
    }
    
    public function login(Request $request, Response $response)
    {
        $accessKey = $request->getRow()['accessKey'];
        $password = $request->getRow()['password'];

        $isLogginInWithEmail = filter_var($accessKey, FILTER_VALIDATE_EMAIL);
        
        $user_BD;
        $params = [
            "CDSENHAOPERWEB" => md5($password)    
        ];
        if($isLogginInWithEmail){
            $params["DSEMAILOPER"] = $accessKey;
            $user_BD = $this->getUserByEmailAndPass($params);
        } else {
            $params["NMACESSO"] = $accessKey;
            $user_BD = $this->getUserByAccessNameAndPass($params);
        }

        if (!empty($user_BD)) {
            $newConnection = $this->connectionCustom->updateVpdConnection($user_BD[0]['NRORG']);

            if ($newConnection) {
                $this->entityManager = $newConnection;
                
                if($isLogginInWithEmail){
                    $user_BD = $this->getUserByEmailAndPass($params, $user_BD[0]['NRORG'], $user_BD[0]['CDOPERADOR']);
                } else {
                    $user_BD = $this->getUserByAccessNameAndPass($params, $user_BD[0]['NRORG'], $user_BD[0]['CDOPERADOR']);
                }
            }
        }

        $fileServerFilesPath = $this->fileServerFilesPath;
        if(strpos($fileServerFilesPath, '/files/files/') !== false){
            $fileServerFilesPath = str_replace  ('/files/files/', '', $this->fileServerFilesPath)."/files";
        }

        $msg = "Usuário e/ou senhas inválidos!";
        if (count($user_BD) > 0) {
            $user_BD = $user_BD[0];
            $userData = [
                "CDOPERADOR" => $user_BD['CDOPERADOR'],
                "CDFILIAL" => $user_BD['CDFILIALSELECIONADA'],
                "NRORG" => $user_BD['NRORG'],
                "fileServiceFilesPath" => $fileServerFilesPath
            ];
            $token = JWT::generateToken($userData);
            $response->addDataSet(new DataSet('token', [$token]));
            $response->addDataSet(new DataSet('userData', $userData));
        } else {
            $response->setError(new Error($msg, "401"));
        }
    }

    public function testConection(Request $request, Response $response)
    {
        return true;
    }

    public function doUpLoadOfLocalData(Request $request, Response $response)
    {
        try {
            if (isset($request->getRow()['respostas'])) {
                $this->loginService->insertASW($request->getRow()['respostas']);
            }

            if (isset($request->getRow()['avaliacoes']) && array_key_exists('NRLOCALPESQ', $request->getRow())) {
                $this->loginService->insertAVA($request->getRow()['avaliacoes'], $request->getRow()['NRLOCALPESQ']);
            }
            
            if (isset($request->getRow()['reservedPlates'])) {
                $this->insertReservedPlates($request->getRow()['reservedPlates'], $request->getRow()['NRLOCALPESQ']);
            }

            if (isset($request->getRow()['reservedServices'])) {
                $this->insertReservedServices($request->getRow()['reservedServices'], $request->getRow()['NRLOCALPESQ']);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParametrizationData(Request $request, Response $response)
    {
        $NRORG = $request->getRow()['NRORG'];
        $CDFILIAL = $request->getRow()['CDFILIAL'];
        $NRLOCALPESQ = isset($request->getRow()['NRLOCALPESQ']) ? $request->getRow()['NRLOCALPESQ'] : null;
        $CDOPERADOR = $request->getRow()['CDOPERADOR'];
        try {
            $appConfig = $this->loginService->getAppConfig($NRORG, $CDFILIAL, $NRLOCALPESQ);
            $cardapio = $this->loginService->getCardData($NRORG, $CDFILIAL, $NRLOCALPESQ, $CDOPERADOR, $request->getRow()['HIDE_CARD_EXIBITIION']);
            $pesquisas = $this->loginService->getPesqData($NRORG, $CDFILIAL, $NRLOCALPESQ, $CDOPERADOR, $request->getRow()['PRESENTATION_MODE']);
            $informativos = $this->loginService->getInforData($NRORG, $CDFILIAL, $NRLOCALPESQ, $CDOPERADOR);
            $turnos = $this->loginService->getTurnoData($NRORG, $NRLOCALPESQ);

            $response->addDataSet(new DataSet("cardapio", $cardapio));
            $response->addDataSet(new DataSet("pesquisas", $pesquisas));
            $response->addDataSet(new DataSet("informativos", $informativos));
            $response->addDataSet(new DataSet("appConfig", $appConfig));
            $response->addDataSet(new DataSet("turnos", $turnos));
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insertReservedPlates($reservedPlates, $NRLOCALPESQ){
        foreach($reservedPlates as $plate){
            $this->loginService->insertReservedPlates($plate, $NRLOCALPESQ);
        }
    }
    
    private function insertReservedServices($reservedServices, $NRLOCALPESQ){
        foreach($reservedServices as $service){
            $this->loginService->insertReservedServices($service, $NRLOCALPESQ);
        }
    }
}
